@@ -1,18 +1,20 @@
 __package__ = 'archivebox.index'
 
 from io import StringIO
+from pathlib import Path
 from typing import List, Tuple, Iterator
+from django.db.models import QuerySet
+from django.db import transaction
 
 from .schema import Link
 from ..util import enforce_types
-from ..config import setup_django, OUTPUT_DIR
+from ..config import OUTPUT_DIR
 
 
 ### Main Links Index
 
 @enforce_types
-def parse_sql_main_index(out_dir: str=OUTPUT_DIR) -> Iterator[Link]:
-    setup_django(out_dir, check_db=True)
+def parse_sql_main_index(out_dir: Path=OUTPUT_DIR) -> Iterator[Link]:
     from core.models import Snapshot
 
     return (
@@ -21,47 +23,59 @@ def parse_sql_main_index(out_dir: str=OUTPUT_DIR) -> Iterator[Link]:
     )
 
 @enforce_types
-def remove_from_sql_main_index(links: List[Link], out_dir: str=OUTPUT_DIR) -> None:
-    setup_django(out_dir, check_db=True)
-    from core.models import Snapshot
-    from django.db import transaction
-
+def remove_from_sql_main_index(snapshots: QuerySet, out_dir: Path=OUTPUT_DIR) -> None:
     with transaction.atomic():
-        for link in links:
-            Snapshot.objects.filter(url=link.url).delete()
+        snapshots.delete()
 
 @enforce_types
-def write_sql_main_index(links: List[Link], out_dir: str=OUTPUT_DIR) -> None:
-    setup_django(out_dir, check_db=True)
+def write_link_to_sql_index(link: Link):
     from core.models import Snapshot
-    from django.db import transaction
+    info = {k: v for k, v in link._asdict().items() if k in Snapshot.keys}
+    tags = info.pop("tags")
+    if tags is None:
+        tags = []
+
+    try:
+        info["timestamp"] = Snapshot.objects.get(url=link.url).timestamp
+    except Snapshot.DoesNotExist:
+        while Snapshot.objects.filter(timestamp=info["timestamp"]).exists():
+            info["timestamp"] = str(float(info["timestamp"]) + 1.0)
+
+    snapshot, _ = Snapshot.objects.update_or_create(url=link.url, defaults=info)
+    snapshot.save_tags(tags)
+    return snapshot
 
-    with transaction.atomic():
-        for link in links:
-            info = {k: v for k, v in link._asdict().items() if k in Snapshot.keys}
-            try:
-                info['timestamp'] = Snapshot.objects.get(url=link.url).timestamp
-            except Snapshot.DoesNotExist:
-                pass
-            Snapshot.objects.update_or_create(url=link.url, defaults=info)
 
 @enforce_types
-def write_sql_link_details(link: Link, out_dir: str=OUTPUT_DIR) -> None:
-    setup_django(out_dir, check_db=True)
+def write_sql_main_index(links: List[Link], out_dir: Path=OUTPUT_DIR) -> None:
+    with transaction.atomic():
+        for link in links:
+            write_link_to_sql_index(link)
+            
+
+@enforce_types
+def write_sql_link_details(link: Link, out_dir: Path=OUTPUT_DIR) -> None:
     from core.models import Snapshot
-    from django.db import transaction
 
     with transaction.atomic():
-        snap = Snapshot.objects.get(url=link.url)
+        try:
+            snap = Snapshot.objects.get(url=link.url)
+        except Snapshot.DoesNotExist:
+            snap = write_link_to_sql_index(link)
         snap.title = link.title
-        snap.tags = link.tags
+
+        tag_set = (
+            set(tag.strip() for tag in (link.tags or '').split(','))
+        )
+        tag_list = list(tag_set) or []
+
         snap.save()
+        snap.save_tags(tag_list)
 
 
 
 @enforce_types
-def list_migrations(out_dir: str=OUTPUT_DIR) -> List[Tuple[bool, str]]:
-    setup_django(out_dir, check_db=False)
+def list_migrations(out_dir: Path=OUTPUT_DIR) -> List[Tuple[bool, str]]:
     from django.core.management import call_command
     out = StringIO()
     call_command("showmigrations", list=True, stdout=out)
@@ -77,8 +91,7 @@ def list_migrations(out_dir: str=OUTPUT_DIR) -> List[Tuple[bool, str]]:
     return migrations
 
 @enforce_types
-def apply_migrations(out_dir: str=OUTPUT_DIR) -> List[str]:
-    setup_django(out_dir, check_db=False)
+def apply_migrations(out_dir: Path=OUTPUT_DIR) -> List[str]:
     from django.core.management import call_command
     null, out = StringIO(), StringIO()
     call_command("makemigrations", interactive=False, stdout=null)
@@ -88,7 +101,6 @@ def apply_migrations(out_dir: str=OUTPUT_DIR) -> List[str]:
     return [line.strip() for line in out.readlines() if line.strip()]
 
 @enforce_types
-def get_admins(out_dir: str=OUTPUT_DIR) -> List[str]:
-    setup_django(out_dir, check_db=False)
+def get_admins(out_dir: Path=OUTPUT_DIR) -> List[str]:
     from django.contrib.auth.models import User
     return User.objects.filter(is_superuser=True)
